unit fDEAOrder;

interface

uses
  Winapi.Windows,
  Winapi.Messages,
  System.Actions,
  System.SysUtils,
  System.Variants,
  System.Classes,
  System.UITypes,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.StdCtrls,
  Vcl.ActnList,
  Vcl.ExtCtrls,
  TRPCB,
  oPKIEncryption;

type
  TfrmDEAOrder = class(TForm)
    edtDEANumber: TEdit;
    cbxIsDEASig: TCheckBox;
    edtIssuanceDate: TEdit;
    edtPatientName: TEdit;
    edtPatientAddress: TEdit;
    edtDrugName: TEdit;
    edtQuantity: TEdit;
    edtDirections: TEdit;
    edtDetoxNumber: TEdit;
    edtProviderName: TEdit;
    edtProviderAddress: TEdit;
    edtOrderNumber: TEdit;
    lblIssuanceData: TLabel;
    lblPatientName: TLabel;
    lblPatientAddress: TLabel;
    lblDrugName: TLabel;
    lblQuantity: TLabel;
    lblDirections: TLabel;
    lblDetoxNumber: TLabel;
    lblProviderName: TLabel;
    lblProviderAddress: TLabel;
    lblDEANumber: TLabel;
    lblOrderNumber: TLabel;
    acList: TActionList;
    acSampleData: TAction;
    acLoadFromVistA: TAction;
    Label1: TLabel;
    acHashOrder: TAction;
    edtHashString: TEdit;
    Label2: TLabel;
    edtHashValue: TEdit;
    acSignOrder: TAction;
    acLoadDEAComponents: TAction;
    memSignature: TMemo;
    lblDigitalSignature: TLabel;
    Panel1: TPanel;
    btnCancel: TButton;
    btnLoadSampleData: TButton;
    btnLoadVistAData: TButton;
    btnHashOrder: TButton;
    btnSignOrder: TButton;
    Bevel1: TBevel;
    acSaveSignature: TAction;
    acValuesHaveChanged: TAction;

    procedure FormCreate(Sender: TObject);
    procedure acListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure acSampleDataExecute(Sender: TObject);
    procedure acLoadFromVistAExecute(Sender: TObject);
    procedure acHashOrderExecute(Sender: TObject);
    procedure acLoadDEAComponentsExecute(Sender: TObject);
    procedure acSignOrderExecute(Sender: TObject);
    procedure acSaveSignatureExecute(Sender: TObject);
    procedure acValuesHaveChangedExecute(Sender: TObject);
  private
    fPKIEncryptionEngine: IPKIEncryptionEngine;
    fPKIEncryptionDataDEAOrder: IPKIEncryptionDataDEAOrder;
    fRPCBroker: TRPCBroker;
  end;

var
  frmDEAOrder: TfrmDEAOrder;

procedure HashDEAOrder(aPKIEncryptionEngine: IPKIEncryptionEngine; aRPCBroker: TRPCBroker = nil);

implementation

{$R *.dfm}


uses
  VAUtils,
  MFunStr,
  IniFiles;

const
  DETOX_MESSAGE = 'Not currently part of Hash/Signature';
  DSIG_SIZE = 50;

procedure HashDEAOrder(aPKIEncryptionEngine: IPKIEncryptionEngine; aRPCBroker: TRPCBroker = nil);
begin
  with TfrmDEAOrder.Create(Application) do
    try
      begin
        if aPKIEncryptionEngine.QueryInterface(IPKIEncryptionEngine, fPKIEncryptionEngine) = 0 then
          begin
            fRPCBroker := aRPCBroker;
            NewPKIEncryptionDataDEAOrder(fPKIEncryptionDataDEAOrder);
            ShowModal;
          end
        else
          ShowMsg('Unable to get PKIEncryptionEngine Interface', 'Error', smiError, smbOK);
      end;
    finally
      Free;
    end;
end;

procedure TfrmDEAOrder.FormCreate(Sender: TObject);
begin
  with GetEnumerator do
    while MoveNext do
      begin
        if Current is TEdit then
          TEdit(Current).Text := '';
        if Current is TCheckBox then
          TCheckBox(Current).Checked := False;
      end;
  edtDetoxNumber.Text := DETOX_MESSAGE;
  Font := Application.MainForm.Font;
  Color := Application.MainForm.Color;
end;

procedure TfrmDEAOrder.acHashOrderExecute(Sender: TObject);
begin
  try
    edtHashString.Text := '';
    edtHashValue.Text := '';
    memSignature.Lines.Clear;

    acLoadDEAComponents.Execute;
    fPKIEncryptionDataDEAOrder.Validate;
    fPKIEncryptionEngine.HashData(fPKIEncryptionDataDEAOrder);
    edtHashString.Text := fPKIEncryptionDataDEAOrder.Buffer;
    edtHashValue.Text := fPKIEncryptionDataDEAOrder.HashText;
    memSignature.Lines.Add('No signature requested.');
  except
    on E: Exception do
      begin
        ShowMsg(Format('Error: %s', [E.Message]), 'Error', smiError, smbOK);
        edtHashString.Text := E.Message;
        edtHashValue.Text := E.Message;
      end;
  end;
end;

procedure TfrmDEAOrder.acSampleDataExecute(Sender: TObject);
begin
  edtDEANumber.Text := 'BR549';
  cbxIsDEASig.Checked := True;
  edtIssuanceDate.Text := 'Now';
  edtPatientName.Text := 'FAKEPATIENT,JOE BLOW';
  edtPatientAddress.Text := '123 ANYSTREET ANYTOWN ANYSTATE 00000-0000';
  edtDrugName.Text := 'TYLENOL 325MG';
  edtQuantity.Text := '300';
  edtDirections.Text := 'PRN FOR PAIN';
  edtDetoxNumber.Text := DETOX_MESSAGE;
  edtProviderName.Text := 'DR. FEEL NO PAIN';
  edtProviderAddress.Text := 'HANGS OUT IN CAFETERIA MOSTLY';
  edtOrderNumber.Text := 'ORDER-101';
end;

procedure TfrmDEAOrder.acSaveSignatureExecute(Sender: TObject);
var
  aSignature: string;
  i: integer;
begin
  with TSaveDialog.Create(Application) do
    try
      if Execute then
        begin
          if FileExists(FileName) then
            DeleteFile(FileName);

          with TIniFile.Create(FileName) do
            try
              WriteString('PKI', 'Hash', fPKIEncryptionDataDEAOrder.HashText);
              WriteString('PKI', 'Data', fPKIEncryptionDataDEAOrder.Buffer);
              WriteString('PKI', 'DateTimeSigned', fPKIEncryptionDataDEAOrder.FMDateTimeSigned);
              aSignature := fPKIEncryptionDataDEAOrder.Signature;
              i := 1;
              while Length(aSignature) > 0 do
                begin
                  WriteString('SIGNATURE', Format('%.3d', [i]), Copy(aSignature, 1, DSIG_SIZE));
                  aSignature := Copy(aSignature, DSIG_SIZE + 1, Length(aSignature));
                  inc(i);
                end;
              UpdateFile;
            finally
              Free;
            end;
        end;
    finally
      Free;
    end;
end;

procedure TfrmDEAOrder.acSignOrderExecute(Sender: TObject);
var
  aDSig: string;
begin
  try
    edtHashString.Text := '';
    edtHashValue.Text := '';
    memSignature.Lines.Clear;

    case VerifyPKIPin(fPKIEncryptionEngine) of
      prOK:
        begin
          Screen.Cursor := crHourGlass;
          acLoadDEAComponents.Execute;
          fPKIEncryptionEngine.SignData(fPKIEncryptionDataDEAOrder);
          edtHashString.Text := fPKIEncryptionDataDEAOrder.Buffer;
          edtHashValue.Text := fPKIEncryptionDataDEAOrder.HashText;
          memSignature.Lines.Clear;
          aDSig := fPKIEncryptionDataDEAOrder.Signature;
          while Length(aDSig) > 0 do
            begin
              memSignature.Lines.Add(Copy(aDSig, 1, DSIG_SIZE));
              aDSig := Copy(aDSig, DSIG_SIZE + 1, Length(aDSig));
            end;
          if ShowMsg('Signing Complete, save results to local disk?', 'Confirm', smiQuestion, smbYesNo) = smrYes then
            acSaveSignature.Execute;
        end;
      prCancel:
        raise EPKIEncryptionError.Create('Signing Canceled.');
      prLocked:
        raise EPKIEncryptionError.Create('Card Locked.');
      prError:
        raise EPKIEncryptionError.Create('Error signing order.');
    end;
  except
    on E: Exception do
      begin
        ShowMsg(Format('Error: %s', [E.Message]), 'Error', smiError, smbOK);
        edtHashString.Text := E.Message;
        edtHashValue.Text := E.Message;
      end;
  end;
  Screen.Cursor := crDefault;
end;

procedure TfrmDEAOrder.acValuesHaveChangedExecute(Sender: TObject);
begin
  edtHashString.Text := 'Values have changed.';
  edtHashValue.Text := 'Values have changed.';
  memSignature.Lines.Text := 'Values have changed.';
end;

procedure TfrmDEAOrder.acLoadDEAComponentsExecute(Sender: TObject);
begin
  with fPKIEncryptionDataDEAOrder do
    try
      DEANumber := edtDEANumber.Text;
      IsDEASig := cbxIsDEASig.Checked;
      IssuanceDate := edtIssuanceDate.Text;
      PatientName := edtPatientName.Text;
      PatientAddress := edtPatientAddress.Text;
      DrugName := edtDrugName.Text;
      Quantity := edtQuantity.Text;
      Directions := edtDirections.Text;
      DetoxNumber := ''; // Currently not used
      ProviderName := edtProviderName.Text;
      ProviderAddress := edtProviderAddress.Text;
      OrderNumber := edtOrderNumber.Text;
    except
      on E: Exception do
        begin
          ShowMsg(Format('Error: %s', [E.Message]), 'Error', smiError, smbOK);
          edtHashString.Text := E.Message;
          edtHashValue.Text := E.Message;
        end;
    end;
end;

procedure TfrmDEAOrder.acLoadFromVistAExecute(Sender: TObject);
var
  aParams: array [0 .. 2] of string;
begin
  aParams[0] := '';
  aParams[1] := '';
  aParams[2] := '';

  if InputQuery('Load Order From Vista', ['Patient DFN:', 'User DUZ:', 'CPRS Order Number:'], aParams) then
    try
      fPKIEncryptionDataDEAOrder.Clear;
      fPKIEncryptionDataDEAOrder.LoadFromVistA(fRPCBroker, aParams[0], aParams[1], aParams[2]);

      edtOrderNumber.Text := fPKIEncryptionDataDEAOrder.OrderNumber;
      cbxIsDEASig.Checked := fPKIEncryptionDataDEAOrder.IsDEASig;
      edtIssuanceDate.Text := fPKIEncryptionDataDEAOrder.IssuanceDate;
      edtPatientName.Text := fPKIEncryptionDataDEAOrder.PatientName;
      edtPatientAddress.Text := fPKIEncryptionDataDEAOrder.PatientAddress;
      edtDetoxNumber.Text := DETOX_MESSAGE;
      edtProviderName.Text := fPKIEncryptionDataDEAOrder.ProviderName;
      edtProviderAddress.Text := fPKIEncryptionDataDEAOrder.ProviderAddress;
      edtDEANumber.Text := fPKIEncryptionDataDEAOrder.DEANumber;
      edtDrugName.Text := fPKIEncryptionDataDEAOrder.DrugName;
      edtQuantity.Text := fPKIEncryptionDataDEAOrder.Quantity;
      edtDirections.Text := fPKIEncryptionDataDEAOrder.Directions;
    except
      ShowMsg('Error retrieving data.', 'Error', smiError, smbOK);
    end;
end;

procedure TfrmDEAOrder.acListUpdate(Action: TBasicAction; var Handled: Boolean);
begin
  acLoadFromVistA.Enabled := Assigned(fRPCBroker);
  acSampleData.Enabled := True;
  Handled := True;
end;

end.
