unit fDEASignature;

interface

uses
  Winapi.Windows,
  Winapi.Messages,
  System.SysUtils,
  System.Variants,
  System.Classes,
  System.UITypes,
  Vcl.Graphics,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.ExtCtrls,
  Vcl.StdCtrls,
  oPKIEncryption,
  IdBaseComponent,
  IdComponent,
  IdTCPConnection,
  IdTCPClient,
  TRPCB;

type
  TfrmDEASignature = class(TForm)
    edtHash: TEdit;
    memSignature: TMemo;
    edtIPAddress: TEdit;
    edtIPPort: TEdit;
    lblHash: TLabel;
    lblSignature: TLabel;
    lblIPAddress: TLabel;
    lblIPPort: TLabel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Label1: TLabel;
    Label2: TLabel;
    IdTCPClient1: TIdTCPClient;
    memLogFile: TMemo;
    Label3: TLabel;
    edtData: TEdit;
    Label4: TLabel;
    Panel1: TPanel;
    Bevel3: TBevel;
    btnSend: TButton;
    btnCancel: TButton;
    btnLoadFromVistA: TButton;
    btnLoadFromFile: TButton;
    procedure IdTCPClient1Connected(Sender: TObject);
    procedure IdTCPClient1Status(ASender: TObject; const AStatus: TIdStatus; const AStatusText: string);
    procedure btnLoadFromFileClick(Sender: TObject);
    procedure btnSendClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    fRPCBroker: TRPCBroker;
    procedure WriteData(aData: string; aAppendCRFL: boolean = False);
    function ReadData: string;

    procedure SendData;
    procedure SendTurn;
  public
    { Public Declarations }
  end;

procedure SendDEASignatureToPKIServer(aRPCBroker: TRPCBroker);

implementation

{$R *.dfm}


uses
  VAUtils,
  IdStack,
  IdContext,
  IdTCPServer,
  IniFiles;

procedure SendDEASignatureToPKIServer(aRPCBroker: TRPCBroker);
begin
  with TfrmDEASignature.Create(Application) do
    try
      fRPCBroker := aRPCBroker;
      edtHash.Text := '';
      edtData.Text := '';
      memSignature.Lines.Clear;
      memLogFile.Lines.Clear;

      edtIPAddress.Text := '127.0.0.1';
      edtIPPort.Text := '10265';
      ShowModal;
    finally
      Free;
    end;
end;

procedure TfrmDEASignature.FormCreate(Sender: TObject);
begin
  Font := Application.MainForm.Font;
  Color := Application.MainForm.Color;
end;

procedure TfrmDEASignature.btnLoadFromFileClick(Sender: TObject);
var
  i: integer;
begin
  with TOpenDialog.Create(Application) do
    try
      if Execute then
        begin
          with TIniFile.Create(FileName) do
            try
              edtHash.Text := ReadString('PKI', 'Hash', '');
              edtData.Text := ReadString('PKI', 'Data', '');
              memSignature.Lines.Clear;
              i := 1;
              while ReadString('SIGNATURE', Format('%.3d', [i]), '') <> '' do
                begin
                  memSignature.Lines.Add(ReadString('SIGNATURE', Format('%.3d', [i]), ''));
                  inc(i);
                end;
            finally
              Free;
            end;
        end;
    finally
      Free;
    end;
end;

procedure TfrmDEASignature.btnSendClick(Sender: TObject);
begin
  IdTCPClient1.Host := edtIPAddress.Text;
  IdTCPClient1.Port := StrToInt(edtIPPort.Text);
  IdTCPClient1.OnConnected := IdTCPClient1Connected;
  IdTCPClient1.ConnectTimeout := 5000;
  IdTCPClient1.Connect;

  if IdTCPClient1.Connected then
    begin
      SendData;
      SendTurn;
      memLogFile.Lines.Add('Sending QUIT to close out this session');
      WriteData('QUIT', True);
      IdTCPClient1.Disconnect;
    end
  else
    ShowMsg('Could not connect to the specified server', 'Error', smiError, smbOK);
end;

procedure TfrmDEASignature.IdTCPClient1Connected(Sender: TObject);
begin
  // Kernel sends two lines of text back on connect
  ReadData;
  ReadData;
end;

procedure TfrmDEASignature.IdTCPClient1Status(ASender: TObject; const AStatus: TIdStatus; const AStatusText: string);
begin
  memLogFile.Lines.Add(AStatusText);
end;

procedure TfrmDEASignature.WriteData(aData: string; aAppendCRFL: boolean = False);
begin
  memLogFile.Lines.Add('W:' + aData);
  if aAppendCRFL then
    IdTCPClient1.IOHandler.WriteLn(aData)
  else
    IdTCPClient1.IOHandler.Write(aData);
end;

function TfrmDEASignature.ReadData: string;
begin
  Result := IdTCPClient1.IOHandler.ReadLn;
  memLogFile.Lines.Add('R:' + Result);
end;

procedure TfrmDEASignature.SendData;
var
  i: integer;
  x: string;
begin
  WriteData('HELO ' + ExtractFileName(Application.ExeName), True);
  ReadData;
  WriteData('DATA', True);

  WriteData(Format('%.3d', [Length(edtData.Text)]) + edtData.Text);
  WriteData('000');

  for i := 0 to memSignature.Lines.Count - 1 do
    begin
      x := memSignature.Lines[i];
      if Length(x) > 0 then
        WriteData(Format('%.3d', [Length(x)]) + x);
    end;

  WriteData('000');

  WriteData('000'); // There is no valid datetime sent up to the PKIServer

  WriteData('-10');

  { Now we should get a 220 from the server }
  memLogFile.Lines.Add('Waiting on a 220');

  if Copy(ReadData, 1, 3) = '220' then
    memLogFile.Lines.Add('Hooray - There should be some results waiting now')
  else
    memLogFile.Lines.Add('Uh Oh... Didn''t get the expected 220');
end;

procedure TfrmDEASignature.SendTurn;
var
  aResult: string;
  i: integer;
begin
  memLogFile.Lines.Add('Sending the TURN command to get the results');
  WriteData('TURN', True);

  memLogFile.Lines.Add('First ReadData should be 220 ...');
  ReadData;

  memLogFile.Lines.Add('Second ReadData should be ''DATA...''');
  ReadData;

  memLogFile.Lines.Add('Third ReadData should be results of call');
  aResult := ReadData;

  i := StrToIntDef(Copy(aResult, 1, 3), 0);
  memLogFile.Lines.Add('Server Says The Result From PKIEncryption.ValidateSignature is: ' + Copy(aResult, 4, i));
  memLogFile.Lines.Add('Sending back 220 OK');
  WriteData('220 OK', True);
end;

end.
