unit fMain;

interface

uses
  Winapi.Windows,
  Winapi.Messages,
  System.Actions,
  System.UITypes,
  System.SysUtils,
  System.Variants,
  System.Classes,
  Vcl.Controls,
  Vcl.Forms,
  Vcl.Dialogs,
  Vcl.ActnList,
  Vcl.StdCtrls,
  Vcl.ExtCtrls,
  Vcl.ComCtrls,
  oPKIEncryption,
  TRPCB, Vcl.StdActns, Vcl.Menus;

type
  TfrmMain = class(TForm)
    lblEventLog: TLabel;
    memLog: TMemo;
    acList: TActionList;
    acGetSANFromCard: TAction;
    acIsDigitalSignatureAvailable: TAction;
    acClose: TAction;
    acHashPlainText: TAction;
    acValidatePIN: TAction;
    acConnectDisconnect: TAction;
    acValidateSignature: TAction;
    acDisplayProviders: TAction;
    pnlControls: TPanel;
    bvlControls: TBevel;
    btnConnect: TButton;
    btnValidatePIN: TButton;
    btnGetSANFromCard: TButton;
    btnCardReader: TButton;
    btnDisplayProviders: TButton;
    btnHash: TButton;
    btnClose: TButton;
    Button4: TButton;
    acClearLog: TAction;
    pmnLog: TPopupMenu;
    ClearLog1: TMenuItem;
    acPrintLog: TPrintDlg;
    Print1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure acListUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure acValidatePINExecute(Sender: TObject);
    procedure acGetSANFromCardExecute(Sender: TObject);
    procedure acHashPlainTextExecute(Sender: TObject);
    procedure acCloseExecute(Sender: TObject);
    procedure acIsDigitalSignatureAvailableExecute(Sender: TObject);
    procedure acConnectDisconnectExecute(Sender: TObject);
    procedure acDisplayProvidersExecute(Sender: TObject);
    procedure acClearLogExecute(Sender: TObject);
    procedure acPrintLogAccept(Sender: TObject);
  private
    fPKIEncryptionEngine: IPKIEncryptionEngine;
    fRPCBroker: TRPCBroker;
  public
    procedure ApplicationException(Sender: TObject; E: Exception);
    procedure AddToLog(const aMessage: string);
  end;

var
  frmMain: TfrmMain;

implementation

uses
  VAUtils,
  fDEAOrder,
  fDEASignature,
  IniFiles,
  Wcrypt2,
  oPKIEncryptionCertificate;

{$R *.dfm}


procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Font := Screen.IconFont;
  memLog.Lines.Clear;
  fRPCBroker := TRPCBroker.Create(Application);
  Application.OnException := ApplicationException;
end;

procedure TfrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if ShowMsg('Quit ' + Caption + '?', 'Confirmation', smiQuestion, smbYesNo) = smrYes then
    begin
      if fRPCBroker.Connected then
        acConnectDisconnect.Execute;
      if Assigned(fPKIEncryptionEngine) then
        fPKIEncryptionEngine.OnLogEvent := nil;
      CanClose := True;
    end
  else
    CanClose := False;
end;

procedure TfrmMain.AddToLog(const aMessage: string);
begin
  memLog.Lines.Add(DateTimeToStr(Now) + ': ' + aMessage);
end;

procedure TfrmMain.ApplicationException(Sender: TObject; E: Exception);
begin
  ShowMsg(E.Message, 'Error', smiError, smbOK);
  Close;
end;

procedure TfrmMain.acDisplayProvidersExecute(Sender: TObject);
begin
  fPKIEncryptionEngine.DisplayProviders;
end;

procedure TfrmMain.acListUpdate(Action: TBasicAction; var Handled: Boolean);
begin
  acGetSANFromCard.Enabled := Assigned(fPKIEncryptionEngine);
  acIsDigitalSignatureAvailable.Enabled := Assigned(fPKIEncryptionEngine);
  acHashPlainText.Enabled := Assigned(fPKIEncryptionEngine);
  acValidatePIN.Enabled := Assigned(fPKIEncryptionEngine);
  acValidateSignature.Enabled := Assigned(fPKIEncryptionEngine);
  acDisplayProviders.Enabled := Assigned(fPKIEncryptionEngine);
  Handled := True;
end;

procedure TfrmMain.acPrintLogAccept(Sender: TObject);
begin
  ShowMessage('Assume we have everything set I guess');
end;


procedure TfrmMain.acClearLogExecute(Sender: TObject);
begin
  memLog.Lines.Clear;
end;

procedure TfrmMain.acCloseExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.acConnectDisconnectExecute(Sender: TObject);
var
  aParams: array [0 .. 1] of string;
  i: integer;
begin
  if fRPCBroker.Connected then
    begin
      AddToLog('Disconnecting Broker');
      fRPCBroker.Connected := False;
      fPKIEncryptionEngine := nil;
      acConnectDisconnect.Caption := 'Connect ...';
    end
  else
    begin
      if ParamCount > 0 then
        for i := 1 to ParamCount do
          begin
            if Copy(ParamStr(i), 1, 2) = 's=' then
              aParams[0] := Copy(ParamStr(i), 3, Length(ParamStr(i)));
            if Copy(ParamStr(i), 1, 2) = 'p=' then
              aParams[1] := Copy(ParamStr(i), 3, Length(ParamStr(i)));
          end;

      aParams[0] := '127.0.0.1';
      aParams[1] := '9205';

      if InputQuery('Connect To Vista', ['Server:', 'Listener Port:'], aParams) then
        try
          AddToLog(Format('Connecting Broker To %s@%s', [aParams[0], aParams[1]]));
          fRPCBroker.Server := aParams[0];
          fRPCBroker.ListenerPort := StrToInt(aParams[1]);
          fRPCBroker.Connected := True;
        except
          on E: Exception do
            AddToLog('Error Connecting Broker: ' + E.Message);
        end;
    end;

  if fRPCBroker.Connected then
    try
      AddToLog('Broker Connection Successful');
      acConnectDisconnect.Caption := 'Disconnect ...';
      NewPKIEncryptionEngine(fRPCBroker, fPKIEncryptionEngine);
      fPKIEncryptionEngine.OnLogEvent := AddToLog;
    except
      on E: Exception do
        begin
          ShowMsg('Error building Encryption Engine: ' + E.Message, 'Error', smiError, smbOK);
          AddToLog('Broker Not Connected');
          acConnectDisconnect.Caption := 'Connect ...';
          fPKIEncryptionEngine := nil;
        end;
    end;
end;

procedure TfrmMain.acIsDigitalSignatureAvailableExecute(Sender: TObject);
var
  aMessage: string;
begin
  if not IsDigitalSignatureAvailable(fPKIEncryptionEngine, aMessage) then
    ShowMsg(aMessage, 'Error', smiError, smbOK)
  else
    ShowMsg('Digital Signature Is Available!', 'Information', smiInfo, smbOK);
end;

procedure TfrmMain.acGetSANFromCardExecute(Sender: TObject);
begin
  try
    ShowMsg('SAN From Card: ' + fPKIEncryptionEngine.SANFromCard, 'Information', smiInfo, smbOK);
  except
    on E: Exception do
      ShowMsg(E.Message, 'Error', smiError, smbOK);
  end;
end;

procedure TfrmMain.acHashPlainTextExecute(Sender: TObject);
var
  aPKIEncryptionData: IPKIEncryptionData;
  aBuffer: string;
begin
  if InputQuery('Hash Plain Text', 'Text Value: ', aBuffer) then
    if aBuffer <> '' then
      try
        NewPKIEncryptionData(aPKIEncryptionData);
        aPKIEncryptionData.Buffer := aBuffer;
        fPKIEncryptionEngine.HashData(aPKIEncryptionData);
        AddToLog('Hash Value:        ' + String(aPKIEncryptionData.getHashValue));
        AddToLog('Hash Value (HEX):  ' + aPKIEncryptionData.HashHex);
        AddToLog('Hash Value (MIME): ' + aPKIEncryptionData.HashText);
      except
        on E: Exception do
          ShowMsg('Error: ' + E.Message, 'Error', smiError, smbOK);
      end
    else
      ShowMsg('Can''t hash empty buffer', 'Error', smiError, smbOK)
  else
    AddToLog('Operation Cancelled');
end;

procedure TfrmMain.acValidatePINExecute(Sender: TObject);
begin
  case VerifyPKIPIN(fPKIEncryptionEngine) of
    prOK:
      ShowMsg('PIN value successfully validated.', 'Information', smiInfo, smbOK);
    prCancel:
      ShowMsg('Cancelled.', 'Information', smiInfo, smbOK);
    prLocked:
      ShowMsg('CARD LOCKED!!!!', 'Warning', smiWarning, smbOK);
    prError:
      ShowMsg('Error validating PIN.', 'Error', smiError, smbOK);
  end;
end;

end.
